<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ManutencaoDelivery extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $dataSourceManager;
    protected $MessagesProcedure;
    protected $adService;

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $connection, $MessagesProcedure, $adService) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->MessagesProcedure = $MessagesProcedure;
        $this->adService = $adService;
    }

    public function findCia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'planovoom_companhia';
        $this->parameterBagColumns = array("CDFILIAL", "DTHRPLAVOOPV");

        parent::find($request, $response);
    }

    public function exportSAP (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'deliverym_exportacaosap';
        $this->parameterBagColumns = array("CDFILIAL", "NRDELIVERY");

        parent::find($request, $response);
    }

    public function findVooCia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'vootrecho_planovoom_base_base';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "DTHRPLAVOOPV");

        parent::find($request, $response);
    }

    public function findServicosVoo (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'servplano_servivociafili_servicogen_deliverym';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRVOO", "CDTRECHO", "DTHRPLAVOOPV");

        parent::find($request, $response);
    }

    public function findDeliveriesVoo (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'deliverym_obsdeliverym_paramcomis';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRVOO", "CDTRECHO", "DTHRPLAVOOPV");

        parent::find($request, $response);
    }

    public function findServicosDelivery  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'servplanom_servicociafili_servicogen_classe_ciclo_ciafili';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRVOO", "CDTRECHO", "DTHRPLAVOOPV", "NRDELIVERY");

        parent::find($request, $response);
    }

    public function findServicosAdd  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'servicociafili_servicogen_servciclo_classe_servplanom';
        $this->parameterBagColumns = array("CDCIA", "CDFILIAL", "NRDELIVERY");

        parent::find($request, $response);
    }

    public function findOperGrupo  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'opergrupop_operador_paramcomis';
        $this->parameterBagColumns = array("CDFILIAL", "CDOPERADOR");

        parent::find($request, $response);
    }

    public function findOperGrupoExp  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'opergrupop_paramcomis';
        $this->parameterBagColumns = array("CDFILIAL", "CDOPERADOR");

        parent::find($request, $response);
    }

    public function findNritemplansp  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'servplanom_nritemplansp';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRVOO", "CDTRECHO", "DTHRPLAVOOPV");

        parent::find($request, $response);
    }

    public function findMaxPax  (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'previsaopaxm_max';
        $this->parameterBagColumns = array("CDFILIAL", "CDCIA", "NRVOO", "CDTRECHO", "DTHRPLAVOOPV", "CDCLASSE");

        parent::find($request, $response);
    }

    public function saveServico (DTO\Request $request, DTO\Response $response) {
        $this->dataSourceNameToSave = 'servplanom';
        parent::save($request, $response);
    }

    public function saveDeliveriesVoo  (DTO\Request $request, DTO\Response $response){

        $this->dataSourceNameToSave = 'deliverym';
        parent::save($request, $response);

        $row = $request->getDataSet()->getRows()[0];


        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRDELIVERY',
                'operator' => '=',
                'value' => $row['NRDELIVERY']
            )
        );

        $filterCriteria = new FilterCriteria('obsdeliverym', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $row['__is_new'] = empty($dataSet->getRows());
        $this->dataSourceNameToSave = 'obsdeliverym';
        parent::save($request, $response);
    }

    public function validaExportacao($row) {
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'NRDELIVERY',
                'operator' => '=',
                'value' => $row['NRDELIVERY']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('deliverym_exportacaosap', $conditions);
        if(!$dataSet->getRows()) {
            return 'N';
        } else {
            return $dataSet->getRows()[0]['IDDELIEXPSAP'];
        }
    }

    public function cancelaDelivery (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $expSap = $this->validaExportacao($row);

            if($expSap != 'S') {
                $cdfilial = $row['CDFILIAL'];
                $cdcia = $row['CDCIA'];
                $nrvoo = $row['NRVOO'];
                $cdtrecho = $row['CDTRECHO'];
                $dthrplavoopv = $row['DTHRPLAVOOPV'];
                $vrdesccond = $row['VRDESCCOND'];
                $nrdelivery = $row['NRDELIVERY'];
                $cdopcancdeli = $row['CDOPERADOR'];

                $this->CANCELA_DELIVERYM ($cdfilial, $cdcia, $nrvoo, $cdtrecho, $dthrplavoopv, $vrdesccond, $nrdelivery, $cdopcancdeli);

            } else {
                throw new \Exception("ATE25200_DEL_EXPORT_SAP");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedure->cleanMessage($e->getMessage()), 500));
        }
    }

    public function CANCELA_DELIVERYM ($cdfilial, $cdcia, $nrvoo, $cdtrecho, $dthrplavoopv, $vrdesccond, $nrdelivery, $cdopcancdeli){
        $procedure = new StoredProcedure($this->connection, 'CANCELA_DELIVERYM');

        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_CDCIA',        Param::PARAM_INPUT, $cdcia));
        $procedure->addParam(new Param('P_NRVOO',        Param::PARAM_INPUT, $nrvoo));
        $procedure->addParam(new Param('P_CDTRECHO',     Param::PARAM_INPUT, $cdtrecho));
        $procedure->addParam(new Param('P_DTHRPLAVOOPV', Param::PARAM_INPUT, $dthrplavoopv));
        $procedure->addParam(new Param('P_VRDESCCOND',   Param::PARAM_INPUT, $vrdesccond));
        $procedure->addParam(new Param('P_NRDELIVERY',   Param::PARAM_INPUT, $nrdelivery));
        $procedure->addParam(new Param('P_CDOPCANCDELI', Param::PARAM_INPUT, $cdopcancdeli));

        $procedure->execute();
    }

    public function calculaDelivery (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $expSap = $this->validaExportacao($row);

            if($expSap != 'S') {
                $cdfilial = $row['CDFILIAL'];
                $nrdelivery = $row['NRDELIVERY'];
                $cdcia = $row['CDCIA'];
                $nrvoo = $row['NRVOO'];
                $cdtrecho = $row['CDTRECHO'];
                $dthrplavoopv = $row['DTHRPLAVOOPV'];
                $cdoperador = $this->environment->getCdOperador();
                $cdagrdeco = NULL;
                $cdlocafatcia = NULL;
                $nrseqdeliver = NULL;

                $this->CALCULA_DELIVERYM($nrdelivery, $cdfilial, $cdcia, $nrvoo, $cdtrecho, $dthrplavoopv, $cdagrdeco, $cdlocafatcia, $nrseqdeliver, $cdoperador);
            } else {
                throw new \Exception("ATE25200_DEL_EXPORT_SAP");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedure->cleanMessage($e->getMessage()), 500));
        }
    }

    public function CALCULA_DELIVERYM($nrdelivery, $cdfilial, $cdcia, $nrvoo, $cdtrecho, $dthrplavoopv, $cdagrdeco, $cdlocafatcia, $nrseqdeliver, $cdoperador){

        $procedure = new StoredProcedure($this->connection, 'CALCULA_DELIVERYM');
        $procedure->addParam(new Param('NRDELIVERY',      Param::PARAM_INPUT, $nrdelivery));
        $procedure->addParam(new Param('CDFILIAL',        Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('CDCIA',           Param::PARAM_INPUT, $cdcia));
        $procedure->addParam(new Param('NRVOO',           Param::PARAM_INPUT, $nrvoo));
        $procedure->addParam(new Param('CDTRECHO',        Param::PARAM_INPUT, $cdtrecho));
        $procedure->addParam(new Param('DTHRPLAVOOPV',    Param::PARAM_INPUT, $dthrplavoopv));
        $procedure->addParam(new Param('CDAGRDECO',       Param::PARAM_INPUT, null, null, 20));
        $procedure->addParam(new Param('CDLOCAFATCIA',    Param::PARAM_INPUT, null, null, 20));
        $procedure->addParam(new Param('NRSEQDELIVER',    Param::PARAM_INPUT, null, null, 20));
        $procedure->addParam(new Param('CDOPERADOR',      Param::PARAM_INPUT, $cdoperador));
        $procedure->addParam(new Param('PESOTOTSERV',     Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('TOTALGERALSERV',  Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('VRTAXA',          Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('DESCNOTA',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('DESCSERV',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('DESCDESV',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('PEDESCDESV',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('DESCFIXO',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('PEDESCFIXO',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('DESCCOND',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('PEDESCCOND',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('TOTSBYUT',        Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('TOTSBYNUT',       Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('VRTOTDESC',       Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('VRUTIDESC',       Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDDESCDESV',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDDESCCOND',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDDESCFIXO',      Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDINCDESDESV',    Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDINCDESFIXO',    Param::PARAM_OUTPUT, null, null, 20));
        $procedure->addParam(new Param('IDINCDESCOND',    Param::PARAM_OUTPUT, null, null, 20));

        $result = $procedure->execute();

        $arrayResult = array();
        $arrayResult['PESOTOTSERV'] = $result['PESOTOTSERV'];
        $arrayResult['TOTALGERALSERV'] = $result['TOTALGERALSERV'];
        $arrayResult['VRTAXA'] = $result['VRTAXA'];
        $arrayResult['DESCNOTA'] = $result['DESCNOTA'];
        $arrayResult['DESCSERV'] = $result['DESCSERV'];
        $arrayResult['DESCDESV'] = $result['DESCDESV'];
        $arrayResult['PEDESCDESV'] = $result['PEDESCDESV'];
        $arrayResult['DESCFIXO'] = $result['DESCFIXO'];
        $arrayResult['PEDESCFIXO'] = $result['PEDESCFIXO'];
        $arrayResult['DESCCOND'] = $result['DESCCOND'];
        $arrayResult['PEDESCCOND'] = $result['PEDESCCOND'];
        $arrayResult['TOTSBYUT'] = $result['TOTSBYUT'];
        $arrayResult['TOTSBYNUT'] = $result['TOTSBYNUT'];
        $arrayResult['VRTOTDESC'] = $result['VRTOTDESC'];
        $arrayResult['VRUTIDESC'] = $result['VRUTIDESC'];
        $arrayResult['IDDESCDESV'] = $result['IDDESCDESV'];
        $arrayResult['IDDESCCOND'] = $result['IDDESCCOND'];
        $arrayResult['IDDESCFIXO'] = $result['IDDESCFIXO'];
        $arrayResult['IDINCDESDESV'] = $result['IDINCDESDESV'];
        $arrayResult['IDINCDESFIXO'] = $result['IDINCDESFIXO'];
        $arrayResult['IDINCDESCOND'] = $result['IDINCDESCOND'];

        return $arrayResult;
    }


    public function calculaPreco (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $expSap = $this->validaExportacao($row);

            if($expSap != 'S') {
                $rowCalculaPrecom = array();
                $rowCalculaPrecom['IDINDICE'] = 'N';
                $rowCalculaPrecom['CDFILIAL'] = $row['CDFILIAL'];
                $rowCalculaPrecom['CDCIA'] = $row['CDCIA'];
                $rowCalculaPrecom['NRVOO'] = $row['NRVOO'];
                $rowCalculaPrecom['CDTRECHO'] = $row['CDTRECHO'];
                $rowCalculaPrecom['NRSERVICOCIAFILI'] = $row['NRSERVICOCIAFILI'];
                $rowCalculaPrecom['CDCLASSE'] = $row['CDCLASSE'];
                $rowCalculaPrecom['CDCICLO'] = $row['CDCICLO'];
                $rowCalculaPrecom['QTSERFATSP'] = $row['QTATUAPRODSP'];
                $rowCalculaPrecom['DTHRPLAVOOPV'] = $row['DTHRPLAVOOPV'];
                $rowCalculaPrecom['DTHRABASTPV'] = $row['DTHRABASTPV'];
                $rowCalculaPrecom['QTTOTPAX_PAXSBY'] = $row['QTTOTPAX_PAXSBY'];
                $rowCalculaPrecom['IDCALCPROPDLCOM'] = $row['IDCALCPROPDLCOM'];

                $response->addDataset(new DataSet('response', $this->CALCULA_PRECOM($rowCalculaPrecom)));
            } else {
                throw new \Exception("ATE25200_DEL_EXPORT_SAP");
            }
        } catch (\Exception $e) {
           $response->setError(new Error($this->MessagesProcedure->cleanMessage($e->getMessage()), 500));
        }
    }

    public function CALCULA_PRECOM($rowCalculaPrecom) {
        $procedure = new StoredProcedure($this->connection, 'CALCULA_PRECOM');
        $procedure->addParam(new Param('P_IDINDICE',            Param::PARAM_INPUT, $rowCalculaPrecom['IDINDICE']));
        $procedure->addParam(new Param('P_CDFILIAL',            Param::PARAM_INPUT, $rowCalculaPrecom['CDFILIAL']));
        $procedure->addParam(new Param('P_CDCIA',               Param::PARAM_INPUT, $rowCalculaPrecom['CDCIA']));
        $procedure->addParam(new Param('P_NRVOO',               Param::PARAM_INPUT, $rowCalculaPrecom['NRVOO']));
        $procedure->addParam(new Param('P_CDTRECHO',            Param::PARAM_INPUT, $rowCalculaPrecom['CDTRECHO']));
        $procedure->addParam(new Param('P_NRSERVICOCIAFILI',    Param::PARAM_INPUT, $rowCalculaPrecom['NRSERVICOCIAFILI']));
        $procedure->addParam(new Param('P_CDCLASSE',            Param::PARAM_INPUT, $rowCalculaPrecom['CDCLASSE']));
        $procedure->addParam(new Param('P_CDCICLO',             Param::PARAM_INPUT, $rowCalculaPrecom['CDCICLO']));
        $procedure->addParam(new Param('P_DTHRPLAVOOPV',        Param::PARAM_INPUT, $rowCalculaPrecom['DTHRPLAVOOPV']));
        $procedure->addParam(new Param('P_DTHRABASTPV',         Param::PARAM_INPUT, $rowCalculaPrecom['DTHRABASTPV']));
        $procedure->addParam(new Param('P_QTTOTPAX_PAXSBY',     Param::PARAM_INPUT, $rowCalculaPrecom['QTTOTPAX_PAXSBY']));
        $procedure->addParam(new Param('P_QTSERFATSP',          Param::PARAM_INPUT, $rowCalculaPrecom['QTSERFATSP']));
        $procedure->addParam(new Param('IDCALCPROPDLCOM',       Param::PARAM_INPUT, $rowCalculaPrecom['IDCALCPROPDLCOM']));
        $procedure->addParam(new Param('P_VRPRECORRENTE',       Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_NULL,10));
        $procedure->addParam(new Param('P_VRPREDOLAR',          Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_NULL,10));
        $result = $procedure->execute();
        $arrayResult = array();
        $arrayResult['VRPRECORRENTE'] = $result['P_VRPRECORRENTE'];
        $arrayResult['VRPREDOLAR'] = $result['P_VRPREDOLAR'];
        return $arrayResult;
    }


    public function retorna_agrupamento (DTO\Request\Row $request, DTO\Response $response) {
        try {

            $row = $request->getRow();
            $cdfilial = $row['CDFILIAL'];
            $cdcia = $row['CDCIA'];
            $nrvoo = $row['NRVOO'];
            $cdtrecho = $row['CDTRECHO'];
            $cdclasse = $row['CDCLASSE'];
            $cdequip = $row['CDEQUIP'];
            $sgprefixo = $row['SGPREFIXO'];

            $response->addDataset(new DataSet('response', $this->RETORNA_AGRUPAMENTOM($cdfilial, $cdcia, $nrvoo, $cdtrecho,
            $cdclasse, $cdequip, $sgprefixo)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function RETORNA_AGRUPAMENTOM($cdfilial, $cdcia, $nrvoo, $cdtrecho, $cdclasse, $cdequip, $sgprefixo){
        $procedure = new StoredProcedure($this->connection, 'RETORNA_AGRUPAMENTO');

        $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_CDCIA',     Param::PARAM_INPUT, $cdcia));
        $procedure->addParam(new Param('P_NRVOO',     Param::PARAM_INPUT, $nrvoo));
        $procedure->addParam(new Param('P_CDTRECHO',  Param::PARAM_INPUT, $cdtrecho));
        $procedure->addParam(new Param('P_CDCLASSE',  Param::PARAM_INPUT, $cdclasse));
        $procedure->addParam(new Param('P_CDEQUIP',   Param::PARAM_INPUT, $cdequip));
        $procedure->addParam(new Param('P_SGPREFIXO', Param::PARAM_INPUT, $sgprefixo));
        $procedure->addParam(new Param('P_CDAGRDECO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_NULL,10));

        $result = $procedure->execute();

        $arrayResult = array();
        $arrayResult['CDAGRDECO'] = $result['P_CDAGRDECO'];

        return $arrayResult;
    }

    public function autenticarPorAd($request, $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $this->adService->login($filter[0]['value'], $filter[1]['value']);
            $response->addDataSet(new DataSet('userValid', array(true)));
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}